"""           PARALLEL SERPENTINES - with mesh refinement near walls

This script creates geometry and mesh for a flow field with parallel serpentines and porous transport layer (GDL).
The program SALOME is required to run this script. SALOME is available from https://www.salome-platform.org
Tested with SALOME 9.3.0 and 9.5.0.

 - before you start: adjust the PATH in 'mypath' to point to the localation of the folder 'shapes' on your local drive
 - creates the geometry and mesh -> see block  ### INPUT PARAMETERS from users ###
 - automatically assigns face groups and volume groups for boundary conditions
 - ATTENTION: the value of 'tolerance' in block MESH may need to be adjusted to the mesh resolution, 
   because the sub-meshes are merged and identical points need to be identified.

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

mypath = "C:/Users/supplement/par-serpentine/shapes/"

import sys, math, time
sys.path.insert( 0, mypath)
start_time = time.time()
inputgeo = []	# containers for input variables
inputmesh = []
allgeolist = []	# containers for garbage collection
allmeshlist = []

#            ###################################
#            ### INPUT PARAMETERS from users ###
#            ###################################

# GEOMETRY 

inputgeo.append(0.0) # origin coordinate x	/ 0
inputgeo.append(0.0) # origin coordinate y	/ 1
inputgeo.append(0.0) # origin coordinate z	/ 2
inputgeo.append(0.90e-3)		# channel width in m		/ 3
inputgeo.append(0.338e-3)		# channel height in m		/ 4
inputgeo.append(20.0e-3)    # flow field total length in m	/ 5
inputgeo.append(1.0)		# channel/ rib ratio		/ 6
inputgeo.append(0.3e-3)		# thicknes of porous transport layer (GDL) m	/ 7
inputgeo.append(2)			# number of U-turns			/ 8
inputgeo.append(3)			# number of channels		/ 9
inputgeo.append(5.0e-3)    # additional inlet / outlet in m	/ 10

# MESH

inputmesh.append(4) 	# half number of segments per channel cross section	
inputmesh.append(3) 	# half of GDL height 
tolerance = 1.5 * float( inputgeo[3] / inputmesh[0] ) / (4.0 * 10.0)

#############################################################################################
# END INPUT PARAMETERS from users


# Names and lists for GROUP definition

#            [  0          1           2       3        4     ]
myfacenames= [ 'wall', 'membrane', 'inlet', 'outlet', 'pwall' ]
myfacegroups = [ [],[],[],[],[] ] # this must be the same number as myfacenames 
myvolnames = ['porous', 'chan']  # list of volume names


### intitialise Salome
import salome
salome.salome_init()
import GEOM
from salome.geom import geomBuilder
geompy = geomBuilder.New()
import SMESH
from salome.smesh import smeshBuilder
smesh = smeshBuilder.New()

import SerpentineGrad0, SerpentineGrad1, SerpRibGrad1

# Built the mesh
print("   Starting script ...")

# calculate active area
a = inputgeo[9] + inputgeo[9] * inputgeo[8]
chan = float(a) * inputgeo[3] + float(a) * inputgeo[3] / inputgeo[6]
chan = chan * inputgeo[5] * 10000.0
print("Active area is about  %.1f cm2." % chan)

print("\n Now calculating flow plate ...")
mySerpMesh0 = SerpentineGrad0.CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolnames[1], allgeolist, allmeshlist, tolerance)

# now make porous layer
inputgeo[2] = inputgeo[2] + inputgeo[4] 

mySerpMesh1 = SerpentineGrad1.CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolnames[0], allgeolist, allmeshlist, tolerance)

myRibMesh1 = SerpRibGrad1.CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolnames[0], allgeolist, allmeshlist, tolerance)

MyCompound = smesh.Concatenate( [mySerpMesh0, mySerpMesh1, myRibMesh1], True, True, tolerance, False, name='FFplate')
#                                         meshlist, unite groups, merge nodes, tolerance, force groups creation, name

# cleaning up the study 
sb = salome.myStudy.NewBuilder()

for a in allgeolist:
	meshSO = salome.ObjectToSObject(a)
	sb.RemoveObjectWithChildren( meshSO )
	
for a in allmeshlist:
	meshSO = salome.ObjectToSObject(a)
	sb.RemoveObjectWithChildren( meshSO )

salome.sg.updateObjBrowser()
print("   ... done in %.1f seconds." % (time.time() - start_time) )