"""   Subunit:  SerpentineGrad0

 - creates channels for a parallel serpentine flow field
 - additional length of channel for inlet / outlet is possible
 - basic shapes (objects) are loaded from separate files
 - intermediate shapes are removed from the study
   (see end of this file, can be commented out in order to display all subshapes for debugging)
 - assigns face groups and one volume group
 - requires: BoxGrad.py and BendGrad.py

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

#############################################################################################
### FUNCTION to make geometry   -> with inlet / outlet region
#############################################################################################
def CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolname,  allgeolist, allmeshlist, tolerance):
	#: Create the serpentine 
	# info:  if subshapes are not on display in the SALOME study, the face groups will not be copied to the mesh
	x0 = inputgeo[0]
	y0 = inputgeo[1]
	z0 = inputgeo[2]
	cw = inputgeo[3] 	# channel width 
	ch = inputgeo[4]	# channel height
	mygroups = myfacegroups
	
	geolist = []	
	meshlist = []
	mybox = []
	obc = 0 # object counter
	mydisplayname = 'Serpentine 0'
	segdelta = 0.000000001   # this is added in the integer conversion of number of segments (mesh) for safety reasons
	totalturns = inputgeo[8] + 1
	
	## load basic geometry modules ##
	import BoxGrad, BendGrad
	
	#################################
	### here are some internal lengths for the overall design
	# -> rib = cw / inputgeo[6] and half rib = 0.5 * cw / inputgeo[6]
	inrad = 0.5 * cw / inputgeo[6]  # half rib
	outrad = cw + inrad 			# outer radius and: length of one bend in x- and y-direction
	myangle = 90
	#myres = float( inputmesh[0] / cw)	# mesh resolution
	#seggrad = float( cw / inputmesh[0] )
	myres = float( inputmesh[0] / cw) * 2.0
	seggrad = float( ch / inputmesh[1] )
	chan = inputgeo[9] - 1
	mylen = inputgeo[5] - 2.0 * outrad - float(chan) * (cw + cw/ inputgeo[6] )  # length of box from width of flowfield
	bendrecsize = outrad + inrad	# size of Bendrec unit
	arcseg = 2 * int( bendrecsize * myres  + segdelta )
	
	########################
	### START horizontal ###
	########################
	myturns = 0
	while myturns < totalturns:
		chan = 0
		while chan < inputgeo[9]:
			a = float(chan) * (cw + cw/ inputgeo[6] )
			x0 = inputgeo[0] + a + float( inputgeo[9] * myturns) * (cw + cw/ inputgeo[6] )
			a = (cw + cw/ inputgeo[6] )
			y0 = inputgeo[1] + outrad + a * float(inputgeo[9] - chan - 1)
			
			# channel row
			rot = 0
			mybox = []
			# mesh
			seglen = int( myres * mylen + segdelta) # segments in flow direction (y-direction)
			#
			BoxGrad.DefPts(geompy, math, cw, mylen , ch, x0, y0, z0, 0, mybox)
			myobject = BoxGrad.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			# creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BoxFaces = [ bottom, top, front, back, left, right ]
			faces = [0, -1, -1, -1, 0, 0]
			grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			#createmesh:
			hexa = smesh.Mesh(myobject) 
			BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list	
			obc = obc + 1
			
			chan = chan + 1
		myturns = myturns + 2
	
	myturns = 1
	while myturns < totalturns:
		chan = 0
		while chan < inputgeo[9]:
			a = float(chan) * (cw + cw/ inputgeo[6] )
			x0 = inputgeo[0] + a + float( inputgeo[9] * myturns) * (cw + cw/ inputgeo[6] )
			y0 = inputgeo[1] + outrad + a 
			
			# channel row
			rot = 0
			mybox = []
			# mesh
			seglen = int( myres * mylen + segdelta) # segments in flow direction (y-direction)
			#
			BoxGrad.DefPts(geompy, math, cw, mylen , ch, x0, y0, z0, 0, mybox)
			myobject = BoxGrad.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			#creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BoxFaces = [ bottom, top, front, back, left, right ]
			faces = [0, -1, -1, -1, 0, 0]
			grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			#createmesh:
			hexa = smesh.Mesh(myobject) 
			BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list	
			obc = obc + 1
			
			chan = chan + 1
		myturns = myturns + 2
	######################
	### END horizontal ###
	######################
	
	
	####################
	### START bends  ###
	####################
	myturns = 0
	while myturns < totalturns:
		if myturns < ( totalturns - 1):
			chan = 0
			while chan < inputgeo[9]:
				a = float(chan) * (cw + cw/ inputgeo[6] )
				x0 = inputgeo[0] + a + float( inputgeo[9] * myturns) * (cw + cw/ inputgeo[6] )
				a = (cw + cw/ inputgeo[6] )
				y0 = inputgeo[1] + outrad + a * float(inputgeo[9] - chan - 1)
				
				rot = 90
				y0 = y0 + mylen
				x0 = x0 + outrad
				mybox = []		
				BendGrad.DefPts(geompy, math, outrad, ch, inrad, x0, y0, z0, myangle, rot, mybox)		# create points for bend
				myobject = BendGrad.Make(geompy, mybox)
				test = mydisplayname + str(obc)
				geompy.addToStudy(myobject, test)
				geolist.append(myobject)
				# creategroups:	
				for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
					del a[0:]
				# BendFaces = [ bottom, top, outer, inner, front, back ]
				faces = [0, -1, 0, 0, -1, -1]
				grouplist = BendGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
				#createmesh:
				hexa = smesh.Mesh(myobject) 
				BendGrad.Mesh(geompy, mybox, myobject, hexa, arcseg, seggrad)	# compute mesh
				for a in grouplist:		# copy groups from geometry
					hexa.GroupOnGeom(a)
				meshlist.append(hexa)			# collect submesh in list
				obc = obc + 1
				
				chan = chan + 1
				
				if chan < inputgeo[9]:		# vertical box
					mylen2 = float( inputgeo[9] - chan) * 2.0 * cw + float( inputgeo[9] - chan ) * 2.0 * cw / inputgeo[6]
					y0 = y0 + inrad
					seglen = int( myres * mylen2  + segdelta) # segments in flow direction
					#
					rot = 90
					myx0 = x0 + mylen2
					mybox = []
					BoxGrad.DefPts(geompy, math, cw, mylen2, ch, myx0, y0, z0, rot, mybox)
					myobject = BoxGrad.Make(geompy, mybox)
					test = mydisplayname + str(obc)
					geompy.addToStudy(myobject, test)
					geolist.append(myobject)
					# creategroups:	
					for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
						del a[0:]
					# BoxFaces = [ bottom, top, front, back, left, right ]
					faces = [0, -1, -1, -1, 0, 0]
					grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
					#createmesh:
					hexa = smesh.Mesh(myobject) 
					BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
					for a in grouplist:		# copy groups from geometry
						hexa.GroupOnGeom(a)
					meshlist.append(hexa)			# collect submesh in list	
					obc = obc + 1
		
		if myturns > 0:
			chan = 0
			while chan < inputgeo[9]:
				a = float(chan) * (cw + cw/ inputgeo[6] )
				x0 = inputgeo[0] + a + float( inputgeo[9] * myturns) * (cw + cw/ inputgeo[6] )
				a = (cw + cw/ inputgeo[6] )
				y0 = inputgeo[1] + outrad + a * float(inputgeo[9] - chan - 1)
				
				rot = 270
				x0 = x0 - inrad
				mybox = []		
				BendGrad.DefPts(geompy, math, outrad, ch, inrad, x0, y0, z0, myangle, rot, mybox)		# create points for bend
				myobject = BendGrad.Make(geompy, mybox)
				test = mydisplayname + str(obc)
				geompy.addToStudy(myobject, test)
				geolist.append(myobject)
				# creategroups:	
				for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
					del a[0:]
				# BendFaces = [ bottom, top, outer, inner ]
				faces = [0, -1, 0, 0, -1, -1]
				grouplist = BendGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
				#createmesh:
				hexa = smesh.Mesh(myobject) 
				BendGrad.Mesh(geompy, mybox, myobject, hexa, arcseg, seggrad)	# compute mesh
				for a in grouplist:		# copy groups from geometry
					hexa.GroupOnGeom(a)
				meshlist.append(hexa)			# collect submesh in list
				obc = obc + 1
				
				chan = chan + 1
				
		
		myturns = myturns + 2
	
	myturns = 1
	while myturns < totalturns:
		chan = 0
		while chan < inputgeo[9]:
			a = float(chan) * (cw + cw/ inputgeo[6] )
			x0 = inputgeo[0] + a + float( inputgeo[9] * myturns) * (cw + cw/ inputgeo[6] )
			y0 = inputgeo[1] + outrad + a 
					
			rot = 0
			y0 = y0 + mylen
			x0 = x0 - inrad
			mybox = []		
			BendGrad.DefPts(geompy, math, outrad, ch, inrad, x0, y0, z0, myangle, rot, mybox)		# create points for bend
			myobject = BendGrad.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			# creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BendFaces = [ bottom, top, outer, inner ]
			faces = [0, -1, 0, 0, -1, -1]
			grouplist = BendGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			#createmesh:
			hexa = smesh.Mesh(myobject) 
			BendGrad.Mesh(geompy, mybox, myobject, hexa, arcseg, seggrad)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list
			obc = obc + 1
			
			chan = chan + 1
		
		if myturns < ( totalturns - 1 ):
			chan = 0
			while chan < inputgeo[9]:
				a = float(chan) * (cw + cw/ inputgeo[6] )
				x0 = inputgeo[0] + a + float( inputgeo[9] * myturns) * (cw + cw/ inputgeo[6] )
				y0 = inputgeo[1] + outrad + a 
				
				rot = 180
				x0 = x0 + ( cw + cw/ inputgeo[6] ) - inrad
				mybox = []		
				BendGrad.DefPts(geompy, math, outrad, ch, inrad, x0, y0, z0, myangle, rot, mybox)		# create points for bend
				myobject = BendGrad.Make(geompy, mybox)
				test = mydisplayname + str(obc)
				geompy.addToStudy(myobject, test)
				geolist.append(myobject)
				# creategroups:	
				for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
					del a[0:]
				# BendFaces = [ bottom, top, outer, inner ]
				faces = [0, -1, 0, 0, -1, -1]
				grouplist = BendGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
				# createmesh:
				hexa = smesh.Mesh(myobject) 
				BendGrad.Mesh(geompy, mybox, myobject, hexa, arcseg, seggrad)	# compute mesh
				for a in grouplist:		# copy groups from geometry
					hexa.GroupOnGeom(a)
				meshlist.append(hexa)			# collect submesh in list
				obc = obc + 1
				
				chan = chan + 1
				
				if chan < inputgeo[9]:		# vertical box
					mylen2 = float( inputgeo[9] - chan) * 2.0 * cw + float( inputgeo[9] - chan ) * 2.0 * cw / inputgeo[6]
					y0 = y0 + inrad - cw - cw / inputgeo[6]
					seglen = int( myres * mylen2  + segdelta) # segments in flow direction
					#
					rot = 90
					myx0 = x0 + mylen2
					mybox = []
					BoxGrad.DefPts(geompy, math, cw, mylen2, ch, myx0, y0, z0, rot, mybox)
					myobject = BoxGrad.Make(geompy, mybox)
					test = mydisplayname + str(obc)
					geompy.addToStudy(myobject, test)
					geolist.append(myobject)
					# creategroups:	
					for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
						del a[0:]
					# BoxFaces = [ bottom, top, front, back, left, right ]
					faces = [0, -1, -1, -1, 0, 0]
					grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
					# createmesh:
					hexa = smesh.Mesh(myobject) 
					BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
					for a in grouplist:		# copy groups from geometry
						hexa.GroupOnGeom(a)
					meshlist.append(hexa)			# collect submesh in list	
					obc = obc + 1
		
		myturns = myturns + 2
	#################
	### END bends ###
	#################	
	
	#################################################
	### START horizontal inlet and outlet channel ###
	#################################################
	x0 = inputgeo[0]
	y0 = inputgeo[1] - 0.5 * cw / inputgeo[6]
	z0 = inputgeo[2]
	
	# inlet region
	chan = 0
	while chan < inputgeo[9]:
		a = float(chan) * (cw + cw/ inputgeo[6] )
		x0 = inputgeo[0] + a 
		a = (cw + cw/ inputgeo[6] )
		y1 = inputgeo[1] + outrad + a * float(inputgeo[9] - chan - 1)
		mylen = y1 - y0  # length of box 
		
		# channel row
		rot = 0
		mybox = []
		# mesh
		seglen = int( myres * mylen + segdelta) # segments in flow direction (y-direction)
		#
		BoxGrad.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
		myobject = BoxGrad.Make(geompy, mybox)
		test = mydisplayname + str(obc)
		geompy.addToStudy(myobject, test)
		geolist.append(myobject)
		# creategroups:	
		for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
			del a[0:]
		# BoxFaces = [ bottom, top, front, back, left, right ]
		if inputgeo[10] > 0.0:
			faces = [0, -1, -1, -1, 0, 0]   # no inlet face, because additional inlet zone
		else:
			faces = [0, -1, 2, -1, 0, 0]   # define inlet face
		grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
		# createmesh:
		hexa = smesh.Mesh(myobject) 
		BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
		for a in grouplist:		# copy groups from geometry
			hexa.GroupOnGeom(a)
		meshlist.append(hexa)			# collect submesh in list	
		obc = obc + 1
		
		chan = chan + 1
	
	# outlet region
	#first we check whether the number of turns is even or not
	b = totalturns - 1
	a = int( b / 2)
	a = a * 2
	if a < b: #  the box is on the same side as inlet
		x0 = inputgeo[0]
		y0 = inputgeo[1] - 0.5 * cw / inputgeo[6]
		z0 = inputgeo[2]
		chan = 0
		while chan < inputgeo[9]:
			myturns = totalturns - 1
			a = float(chan) * (cw + cw/ inputgeo[6] )
			x0 = inputgeo[0] + a + float( inputgeo[9] * myturns ) * (cw + cw/ inputgeo[6] )
			y1 = inputgeo[1] + outrad + a 
			mylen = y1 - y0  # length of box 
			
			rot = 0
			mybox = []
			# mesh
			seglen = int( myres * mylen + segdelta) # segments in flow direction (y-direction)
			#
			BoxGrad.DefPts(geompy, math, cw, mylen , ch, x0, y0, z0, 0, mybox)
			myobject = BoxGrad.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			# creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BoxFaces = [ bottom, top, front, back, left, right ]
			if inputgeo[10] > 0.0:
				faces = [0, -1, -1, -1, 0, 0]	# no outlet, because additional outlet zone
			else:
				faces = [0, -1, 3, -1, 0, 0]	# define outlet
			grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			# createmesh:
			hexa = smesh.Mesh(myobject) 
			BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list	
			obc = obc + 1
			
			chan = chan + 1
	else:				# the box needs to be on the opposite side as inlet
		x0 = inputgeo[0]
		y0 = inputgeo[1]
		z0 = inputgeo[2]
		chan = 0
		while chan < inputgeo[9]:
			myturns = totalturns - 1
			a = float(chan) * (cw + cw/ inputgeo[6] )
			x0 = inputgeo[0] + a + float( inputgeo[9] * myturns ) * (cw + cw/ inputgeo[6] )
			a = (cw + cw/ inputgeo[6] )
			y1 = inputgeo[1] + outrad + a * float(inputgeo[9] - chan - 1)
			myboxlen = inputgeo[5] - 2.0 * outrad - float( inputgeo[9] - 1 ) * (cw + cw/ inputgeo[6] )  # original length of box from width of flowfield
			y0 = y1 + myboxlen
			mylen = inputgeo[5] - y0 + 0.5 * cw / inputgeo[6]
			
			rot = 0
			mybox = []
			# mesh
			seglen = int( myres * mylen + segdelta) # segments in flow direction (y-direction)
			#
			BoxGrad.DefPts(geompy, math, cw, mylen , ch, x0, y0, z0, 0, mybox)
			myobject = BoxGrad.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			# creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BoxFaces = [ bottom, top, front, back, left, right ]
			if inputgeo[10] > 0.0:
				faces = [0, -1, -1, -1, 0, 0]	# no outlet face, because additional inlet outlet
			else:
				faces = [0, -1, -1, 3, 0, 0]
			grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			# createmesh:
			hexa = smesh.Mesh(myobject) 
			BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list	
			obc = obc + 1
			
			chan = chan + 1
	
	###############################################
	### END horizontal inlet and outlet channel ###
	###############################################
	

	#################################################
	### START additional inlet and outlet channel ###
	#################################################
	if inputgeo[10] > 0.0:
		x0 = inputgeo[0]
		y0 = inputgeo[1] - 0.5 * cw / inputgeo[6] - inputgeo[10]		# add additional length for inlet
		z0 = inputgeo[2]
		
		# inlet region
		chan = 0
		while chan < inputgeo[9]:
			a = float(chan) * (cw + cw/ inputgeo[6] )
			x0 = inputgeo[0] + a 
			a = (cw + cw/ inputgeo[6] )
			y1 = inputgeo[1] - 0.5 * cw / inputgeo[6]
			mylen = y1 - y0  # length of box 
			
			# channel row
			rot = 0
			mybox = []
			# mesh
			seglen = int( myres * mylen + segdelta) # segments in flow direction (y-direction)
			#
			BoxGrad.DefPts(geompy, math, cw, mylen, ch, x0, y0, z0, 0, mybox)
			myobject = BoxGrad.Make(geompy, mybox)
			test = mydisplayname + str(obc)
			geompy.addToStudy(myobject, test)
			geolist.append(myobject)
			# creategroups:	
			for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
				del a[0:]
			# BoxFaces = [ bottom, top, front, back, left, right ]
			faces = []
			faces = [0, 0, 2, -1, 0, 0]
			grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
			# createmesh:
			hexa = smesh.Mesh(myobject) 
			BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
			for a in grouplist:		# copy groups from geometry
				hexa.GroupOnGeom(a)
			meshlist.append(hexa)			# collect submesh in list	
			obc = obc + 1
			
			chan = chan + 1
		
		# outlet region
		#first we check whether the number of turns is even or not
		b = totalturns - 1
		a = int( b / 2)
		a = a * 2
		if a < b: #  the box is on the same side as inlet
			x0 = inputgeo[0]
			y0 = inputgeo[1] - 0.5 * cw / inputgeo[6] - inputgeo[10]
			z0 = inputgeo[2]
			chan = 0
			while chan < inputgeo[9]:
				myturns = totalturns - 1
				a = float(chan) * (cw + cw/ inputgeo[6] )
				x0 = inputgeo[0] + a + float( inputgeo[9] * myturns ) * (cw + cw/ inputgeo[6] )
				y1 = inputgeo[1] - 0.5 * cw / inputgeo[6]
				mylen = y1 - y0  # length of box 
				
				rot = 0
				mybox = []
				# mesh
				seglen = int( myres * mylen + segdelta) # segments in flow direction (y-direction)
				#
				BoxGrad.DefPts(geompy, math, cw, mylen , ch, x0, y0, z0, 0, mybox)
				myobject = BoxGrad.Make(geompy, mybox)
				test = mydisplayname + str(obc)
				geompy.addToStudy(myobject, test)
				geolist.append(myobject)
				# creategroups:	
				for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
					del a[0:]
				# BoxFaces = [ bottom, top, front, back, left, right ]
				faces = []
				faces = [0, 0, 3, -1, 0, 0]
				grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
				# createmesh:
				hexa = smesh.Mesh(myobject) 
				BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
				for a in grouplist:		# copy groups from geometry
					hexa.GroupOnGeom(a)
				meshlist.append(hexa)			# collect submesh in list	
				obc = obc + 1
				
				chan = chan + 1
		else:				# the box needs to be on the opposite side as inlet
			x0 = inputgeo[0]
			y0 = inputgeo[1] + inputgeo[5] + 0.5 * cw / inputgeo[6]
			z0 = inputgeo[2]
			chan = 0
			while chan < inputgeo[9]:
				myturns = totalturns - 1
				a = float(chan) * (cw + cw/ inputgeo[6] )
				x0 = inputgeo[0] + a + float( inputgeo[9] * myturns ) * (cw + cw/ inputgeo[6] )
				y1 = y0 + inputgeo[10]
				mylen = y1 - y0 
				
				rot = 0
				mybox = []
				# mesh
				seglen = int( myres * mylen + segdelta) # segments in flow direction (y-direction)
				#
				BoxGrad.DefPts(geompy, math, cw, mylen , ch, x0, y0, z0, 0, mybox)
				myobject = BoxGrad.Make(geompy, mybox)
				test = mydisplayname + str(obc)
				geompy.addToStudy(myobject, test)
				geolist.append(myobject)
				# creategroups:	
				for a in mygroups:		# lists must be deleted, because PYTHON just 'links' the name
					del a[0:]
				# BoxFaces = [ bottom, top, front, back, left, right ]
				faces = []
				faces = [0, 0, -1, 3, 0, 0]
				grouplist = BoxGrad.Groups(geompy, mybox, myobject, mygroups, faces, myfacenames, myvolname)
				# createmesh:
				hexa = smesh.Mesh(myobject) 
				BoxGrad.Mesh(geompy, mybox, myobject, hexa, seglen, seggrad)	# compute mesh
				for a in grouplist:		# copy groups from geometry
					hexa.GroupOnGeom(a)
				meshlist.append(hexa)			# collect submesh in list	
				obc = obc + 1
				
				chan = chan + 1
	###############################################
	### END additional inlet and outlet channel ###
	###############################################
	
	### join all subshapes
	mygeo = geompy.MakePartition( geolist )	
	geompy.addToStudy(mygeo, 'Serpentine0')
	#geolist.append(mygeo)
	
	### now join all meshes
	MyCompound = smesh.Concatenate( meshlist, True, True, tolerance, False, name='Serpentine0')
	#                              meshlist, unite groups, merge nodes, tolerance, force groups creation, name
	# cleaning up the study
	for a in meshlist:
		allmeshlist.append(a)
		
	for a in geolist:
		allgeolist.append(a)
		
	### Alternatively, put subshapes into Folders: (this works only for geometry, not for mesh)
	#MyBasic = geompy.NewFolder('Base Objects')
	#geompy.PutListToFolder(geolist, MyBasic)
	
	return MyCompound
#############################################################################################
### END of FUNCTIONS
#############################################################################################