"""           SEPARATE SERPENTINES with tedrahedrons in half circle segments

This script creates geometry and mesh for a flow field with separate serpentines and porous transport layer (GDL).
The program SALOME is required to run this script. SALOME is available from https://www.salome-platform.org
Tested with SALOME 9.3.0 and 9.5.0.

 - before you start: adjust the PATH in 'mypath' to point to the localation of the folder 'shapes' on your local drive
 - creates the geometry and mesh -> see block  ### INPUT PARAMETERS from users ###
 - automatically assigns face groups and volume groups for boundary conditions
 - ATTENTION: the value of 'tolerance' in block MESH may need to be adjusted to the mesh resolution, 
   because the sub-meshes are merged and identical points need to be identified.

Uwe Reimer, Forschungszentrum Juelich GmbH - IEK-14 / Germany
June 2021

DISCLAIMER
This software is part of the article: 'Design and modelling of metallic bipolar plates for a fuel cell range extender'
published at MDPI/ Energies https://www.mdpi.com/journal/energies/special_issues/Analysis_and_Simulation_of_Fuel_Cells
Articles published in Energies are Open-Access articles distributed under the terms and conditions of the Creative Commons Attribution License (CC BY) - see the journal homepage for details. https://www.mdpi.com/journal/energies/about
This program is free software. It comes without any warranty. You can redistribute it and/or modify it.
Please cite the original work appropriately. 
"""

mypath = "C:/Users/supplement/sep-serpentine/shapes/"

import sys, math, time
sys.path.insert( 0, mypath)
start_time = time.time()
inputgeo = []	# containers for input variables
inputmesh = []
allgeolist = []	# containers for garbage collection
allmeshlist = []

#            ###################################
#            ### INPUT PARAMETERS from users ###
#            ###################################

# GEOMETRY 

inputgeo.append(0.0) # origin coordinate x	/ 0
inputgeo.append(0.0) # origin coordinate y	/ 1
inputgeo.append(0.0) # origin coordinate z	/ 2
inputgeo.append(0.90e-3)		# channel width in m		/ 3
inputgeo.append(0.338e-3)		# channel height in m		/ 4
inputgeo.append(1.0)		# channel/ rib ratio		/ 5
inputgeo.append(0.3e-3)		# thicknes of porous transport layer (GDL) / 6
inputgeo.append(3)			# half of number of U-turns	/ 7
inputgeo.append(3)			# number of channels		/ 8
inputgeo.append(5)			# number of spacers		    / 9
inputgeo.append(3)	        # additional inlet / outlet in number of spacers	/ 10

# MESH

inputmesh.append(6) 	# half number of segments per channel height
inputmesh.append(3) 	# half of GDL height
tolerance = float( inputgeo[6] / inputmesh[0] ) * 0.0001

#          #######################################
#          ### END INPUT PARAMETERS from users ###
#          #######################################

#############################################################################################
# Names and lists for GROUP definition
#            [  0          1           2       3        4     ]
myfacenames= [ 'wall', 'membrane', 'inlet', 'outlet', 'pwall' ]
myfacegroups = [ [],[],[],[],[] ] # this must be the same number as myfacenames 
myvolnames = ['porous', 'chan']  # list of volume names

# intitialise Salome
import salome
salome.salome_init()
import GEOM
from salome.geom import geomBuilder
geompy = geomBuilder.New()
import SMESH
from salome.smesh import smeshBuilder
smesh = smeshBuilder.New()

# load basic unit
import Pillar1, Pillar2, PillarRib2tri      # these are scripts located in '/shapes'

print( "   Starting script ..." )

mw = float( 3 + inputgeo[9] + 1)
mw = mw * inputgeo[3] * inputgeo[8] # width
mw = mw * 1000.0
ml = float( 4.0 * inputgeo[3] * inputgeo[7] )
ml = ml + 2.0 * inputgeo[3] * inputgeo[8] # length
ml = ml * 1000.0
ma = ml * mw / 100.0

print("Flowfield width  = %.2f mm" % mw)
print("Flowfield length = %.2f mm" % ml)
print("Flowfield area   = %.2f cm^2" % ma)

inputgeo[10] = inputgeo[10] + 2 # correction for the GDL covered part

myMesh1 = Pillar1.CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolnames[1], allgeolist, allmeshlist, tolerance)

inputgeo[2] = inputgeo[2] + inputgeo[4]  # next layer

myMesh2 = Pillar2.CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolnames[0], allgeolist, allmeshlist, tolerance)
myMesh3 = PillarRib2tri.CreateGeo(salome, geompy, smesh, math, inputgeo, inputmesh, myfacegroups, myfacenames, myvolnames[0], allgeolist, allmeshlist, tolerance)

MyCompound = smesh.Concatenate( [myMesh1, myMesh2, myMesh3], True, True, tolerance, False, name='FFplate')
#                                         meshlist, unite groups, merge nodes, tolerance, force groups creation, name

# cleaning up the study 
sb = salome.myStudy.NewBuilder()

for a in allgeolist:
	meshSO = salome.ObjectToSObject(a)
	sb.RemoveObjectWithChildren( meshSO )
	
for a in allmeshlist:
	meshSO = salome.ObjectToSObject(a)
	sb.RemoveObjectWithChildren( meshSO )

salome.sg.updateObjBrowser()
print( "   ... done in %.1f seconds." % (time.time() - start_time) )